import React, { useState, useRef, useEffect } from 'react';
import _styled, { keyframes } from 'styled-components';
import orderBy from 'lodash/orderBy';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var Category;

(function (Category) {
  Category[Category["Primary"] = 0] = "Primary";
  Category[Category["Secondary"] = 1] = "Secondary";
})(Category || (Category = {}));

var fadeIn = keyframes(["from{opacity:0;}to{opacity:1;}"]);

var highlightedIdName = 'react-comparison-bar-chart-highlighted-item';

var GreaterThan1Root = _styled.div.withConfig({
  displayName: "Row__GreaterThan1Root",
  componentId: "fpfft1-0"
})(["display:flex;&:hover{background-color:#f1f1f1;}"]);

var LessThan1Root = _styled.div.withConfig({
  displayName: "Row__LessThan1Root",
  componentId: "fpfft1-1"
})(["display:flex;background-color:#f1f1f1;&:hover{background-color:#fff;}"]);

var LabelText = _styled.div.withConfig({
  displayName: "Row__LabelText",
  componentId: "fpfft1-2"
})(["width:100%;font-size:0.65rem;font-size:", ";white-space:nowrap;overflow:hidden;text-overflow:ellipsis;opacity:0;animation:", " 0.15s linear 1 forwards 0.3s;background-color:#fff;"], function (_ref) {
  var $dynamicFont = _ref.$dynamicFont;
  return $dynamicFont;
}, fadeIn);

var Cell = _styled.div.withConfig({
  displayName: "Row__Cell",
  componentId: "fpfft1-3"
})(["transition:height 0.3s ease-in-out;overflow:hidden;display:flex;align-items:center;"]);

var TextCell = _styled(Cell).withConfig({
  displayName: "Row__TextCell",
  componentId: "fpfft1-4"
})(["background-color:#fff;"]);

var BarCell = _styled(Cell).withConfig({
  displayName: "Row__BarCell",
  componentId: "fpfft1-5"
})(["display:flex;"]);

var Range = _styled.div.withConfig({
  displayName: "Row__Range",
  componentId: "fpfft1-6"
})(["height:100%;box-sizing:border-box;display:flex;align-items:center;border-left:solid 1px #333;"]);

var Bar = _styled.div.withConfig({
  displayName: "Row__Bar",
  componentId: "fpfft1-7"
})(["height:70%;transition:width 0.2s ease-in-out;"]);

var Row = function Row(props) {
  var d = props.d,
      rowHeight = props.rowHeight,
      gridHeight = props.gridHeight,
      max = props.max,
      onRowHover = props.onRowHover,
      highlighted = props.highlighted,
      chartWidth = props.chartWidth,
      textWidth = props.textWidth,
      lessThan1 = props.lessThan1,
      centerLineValue = props.centerLineValue;
  var Root = lessThan1 ? LessThan1Root : GreaterThan1Root;
  var style = {
    height: rowHeight,
    backgroundColor: highlighted === d.id ? lessThan1 ? '#fff' : '#f1f1f1' : undefined
  };

  var onMouseMove = function onMouseMove(e) {
    if (onRowHover) {
      onRowHover({
        datum: d,
        mouseCoords: {
          x: e.clientX,
          y: e.clientY
        }
      });
    }
  };

  var onMouseLeave = function onMouseLeave(e) {
    if (onRowHover) {
      onRowHover({
        datum: undefined,
        mouseCoords: {
          x: e.clientX,
          y: e.clientY
        }
      });
    }
  };

  var percent = d.value / max * 100;
  var bar;

  if (percent > centerLineValue) {
    bar = React.createElement(React.Fragment, null, React.createElement(Bar, {
      className: 'react-comparison-bar-chart-bar',
      style: {
        backgroundColor: 'transparent',
        width: centerLineValue + "%",
        transition: 'none'
      }
    }), React.createElement(Bar, {
      className: 'react-comparison-bar-chart-bar',
      style: {
        backgroundColor: d.color,
        width: percent - centerLineValue + "%",
        transitionDelay: '0.3s'
      },
      role: 'graphics-symbol',
      "aria-label": 'Value of ' + d.title + ' is ' + d.value
    }));
  } else if (percent < centerLineValue) {
    bar = React.createElement(React.Fragment, null, React.createElement(Bar, {
      className: 'react-comparison-bar-chart-bar',
      style: {
        backgroundColor: 'transparent',
        width: percent + "%",
        transition: 'none'
      }
    }), React.createElement(Bar, {
      className: 'react-comparison-bar-chart-bar',
      style: {
        backgroundColor: d.color,
        width: centerLineValue - percent + "%",
        transitionDelay: '0.3s'
      },
      role: 'graphics-symbol',
      "aria-label": 'Value of ' + d.title + ' is ' + d.value
    }));
  } else {
    bar = null;
  }

  return React.createElement(Root, null, React.createElement(TextCell, {
    style: _extends({}, style, {
      width: textWidth
    }),
    onMouseMove: onMouseMove,
    onMouseLeave: onMouseLeave
  }, React.createElement(LabelText, {
    className: 'react-comparison-bar-chart-row-label',
    style: {
      textAlign: 'left'
    },
    "$dynamicFont": "clamp(0.5rem, " + gridHeight * 0.04 + "px, 0.9rem)"
  }, d.title)), React.createElement(TextCell, {
    style: _extends({}, style, {
      width: '0.75rem'
    }),
    onMouseMove: onMouseMove,
    onMouseLeave: onMouseLeave
  }), React.createElement(BarCell, {
    id: highlighted === d.id ? highlightedIdName : undefined,
    style: _extends({}, style, {
      width: chartWidth
    }),
    onMouseMove: onMouseMove,
    onMouseLeave: onMouseLeave
  }, React.createElement(Range, {
    style: {
      width: max + "%"
    }
  }, bar)));
};

var Row$1 = React.memo(Row);

var overflowPadding = 1;

var Container = _styled.div.withConfig({
  displayName: "Root__Container",
  componentId: "sc-1cw2hx3-0"
})(["width:100%;height:100%;padding-bottom:2rem;display:flex;flex-direction:column;position:relative;box-sizing:border-box;position:relative;overflow:hidden;"]);

var ChartContainer = _styled.div.withConfig({
  displayName: "Root__ChartContainer",
  componentId: "sc-1cw2hx3-1"
})(["width:100%;height:100%;position:relative;transform:translate(0px);will-change:transform;"]);

var ChartBlock = _styled.div.withConfig({
  displayName: "Root__ChartBlock",
  componentId: "sc-1cw2hx3-2"
})(["grid-column:1 / -1;width:100%;"]);

var TitleRoot = _styled.div.withConfig({
  displayName: "Root__TitleRoot",
  componentId: "sc-1cw2hx3-3"
})(["margin-left:auto;display:flex;position:absolute;top:1px;font-size:0.65rem;font-size:", ";"], function (_ref) {
  var $dynamicFont = _ref.$dynamicFont;
  return $dynamicFont;
});

var ChartOverlayContainer = _styled.div.withConfig({
  displayName: "Root__ChartOverlayContainer",
  componentId: "sc-1cw2hx3-4"
})(["pointer-events:none;margin-left:auto;display:flex;position:absolute;top:1px;"]);

var AxisLines = _styled.div.withConfig({
  displayName: "Root__AxisLines",
  componentId: "sc-1cw2hx3-5"
})(["position:absolute;width:100%;display:flex;transform:translate(0px);will-change:transform;"]);

var Grid = _styled.div.withConfig({
  displayName: "Root__Grid",
  componentId: "sc-1cw2hx3-6"
})(["width:100%;height:100%;grid-row:1;display:grid;grid-template-columns:120px 0.75rem 1fr;position:relative;overflow:auto;overflow-y:scroll;overflow-x:hidden;"]);

var AxisValue = _styled.div.withConfig({
  displayName: "Root__AxisValue",
  componentId: "sc-1cw2hx3-7"
})(["display:flex;flex-shrink:0;background-color:#fff;position:relative;pointer-events:none;&:after{content:'';width:100%;height:0;position:absolute;bottom:0;}&:not(:last-child){&:after{border-bottom:solid 2px #333;z-index:10;}}:last-child{&:after{border-bottom:solid 2px transparent;}}"]);

var AxisText = _styled.span.withConfig({
  displayName: "Root__AxisText",
  componentId: "sc-1cw2hx3-8"
})(["font-size:0.55rem;font-size:", ";transform:translate(-50%,calc(100% + 4px));position:absolute;bottom:0;"], function (_ref2) {
  var $dynamicFont = _ref2.$dynamicFont;
  return $dynamicFont;
});

var AxisLine = _styled.div.withConfig({
  displayName: "Root__AxisLine",
  componentId: "sc-1cw2hx3-9"
})(["position:absolute;top:0;height:100%;width:0;border-left:solid 1px #dfdfdf;"]);

var AxisTitle = _styled.div.withConfig({
  displayName: "Root__AxisTitle",
  componentId: "sc-1cw2hx3-10"
})(["position:absolute;bottom:0;z-index:1;padding:0 0 0.3rem 1rem;box-sizing:border-box;pointer-events:none;transform:translate(-1rem,0);font-weight:600;font-size:0.75rem;color:#333;text-transform:uppercase;@media (max-height:600px){font-size:0.65rem;}"]);

var CenterLine = _styled.div.withConfig({
  displayName: "Root__CenterLine",
  componentId: "sc-1cw2hx3-11"
})(["position:fixed;top:0;bottom:0;width:0;height:100%;border-left:dashed 2px #333;text-align:center;display:flex;justify-content:center;"]);

var CenterLineLabel = _styled.div.withConfig({
  displayName: "Root__CenterLineLabel",
  componentId: "sc-1cw2hx3-12"
})(["font-size:0.5rem;font-size:", ";white-space:nowrap;padding-left:0.7rem;"], function (_ref3) {
  var $dynamicFont = _ref3.$dynamicFont;
  return $dynamicFont;
});

var CenterLineBackground = _styled.div.withConfig({
  displayName: "Root__CenterLineBackground",
  componentId: "sc-1cw2hx3-13"
})(["background-color:#fff;padding:0.1rem;"]);

var BufferRow = _styled.div.withConfig({
  displayName: "Root__BufferRow",
  componentId: "sc-1cw2hx3-14"
})(["z-index:100;display:flex;align-items:flex-end;"]);

var Midline = _styled.div.withConfig({
  displayName: "Root__Midline",
  componentId: "sc-1cw2hx3-15"
})(["border-top:dashed 2px #333;width:100%;position:absolute;background-color:#f1f1f1;z-index:-1;transform:translateY(1px);font-size:0.75rem;"]);

var MidlineOverText = _styled.div.withConfig({
  displayName: "Root__MidlineOverText",
  componentId: "sc-1cw2hx3-16"
})(["font-size:0.55rem;font-size:", ";position:absolute;right:1rem;top:-0.25rem;transform:translateY(-100%);z-index:100;"], function (_ref4) {
  var $dynamicFont = _ref4.$dynamicFont;
  return $dynamicFont;
});

var MidlineUnderText = _styled.div.withConfig({
  displayName: "Root__MidlineUnderText",
  componentId: "sc-1cw2hx3-17"
})(["font-size:0.55rem;font-size:", ";position:absolute;right:1rem;bottom:0;"], function (_ref5) {
  var $dynamicFont = _ref5.$dynamicFont;
  return $dynamicFont;
});

var ScrollDownText = _styled.div.withConfig({
  displayName: "Root__ScrollDownText",
  componentId: "sc-1cw2hx3-18"
})(["font-size:0.65rem;font-size:", ";position:absolute;right:calc(0.25rem + 12px);top:50%;display:flex;justify-content:flex-end;align-items:center;max-width:150px;pointer-events:none;text-shadow:1px 1px 0 #fff,-1px -1px 0 #fff,1px -1px 0 #fff,-1px 1px 0 #fff,1px 1px 0 #fff;"], function (_ref6) {
  var $dynamicFont = _ref6.$dynamicFont;
  return $dynamicFont;
});

var ScrollDownArrow = _styled.div.withConfig({
  displayName: "Root__ScrollDownArrow",
  componentId: "sc-1cw2hx3-19"
})(["margin-right:0.75rem;font-size:1rem;"]);

var Root = function Root(props) {
  var data = props.data,
      formatValue = props.formatValue,
      axisLabel = props.axisLabel,
      onRowHover = props.onRowHover,
      highlighted = props.highlighted,
      onHighlightError = props.onHighlightError,
      numberOfXAxisTicks = props.numberOfXAxisTicks,
      centerLineValue = props.centerLineValue,
      centerLineLabel = props.centerLineLabel,
      overMideLineLabel = props.overMideLineLabel,
      underMideLineLabel = props.underMideLineLabel,
      scrollDownText = props.scrollDownText;

  if (!data.length) {
    return null;
  }

  var _useState = useState({
    gridHeight: 0,
    chartWidth: 0,
    chartHeight: 0,
    textWidth: 0,
    scrollWidth: 0
  }),
      _useState$ = _useState[0],
      gridHeight = _useState$.gridHeight,
      chartWidth = _useState$.chartWidth,
      chartHeight = _useState$.chartHeight,
      textWidth = _useState$.textWidth,
      scrollWidth = _useState$.scrollWidth,
      setMeasurements = _useState[1];

  var rootRef = useRef(null);
  var textRef = useRef(null);
  var chartRef = useRef(null);
  useEffect(function () {
    if (rootRef && rootRef.current && chartRef && chartRef.current && textRef && textRef.current) {
      var chartRect = chartRef.current.getBoundingClientRect();
      var textRect = textRef.current.getBoundingClientRect();

      var _scrollWidth = rootRef.current.offsetWidth - rootRef.current.clientWidth;

      setMeasurements({
        gridHeight: rootRef.current.offsetHeight,
        chartWidth: chartRect.width,
        chartHeight: chartRect.height,
        textWidth: textRect.width,
        scrollWidth: _scrollWidth
      });
    }
  }, [rootRef, chartRef]);
  useEffect(function () {
    var updateWindowWidth = function updateWindowWidth() {
      if (rootRef && rootRef.current && chartRef && chartRef.current && textRef && textRef.current) {
        var chartRect = chartRef.current.getBoundingClientRect();
        var textRect = textRef.current.getBoundingClientRect();

        var _scrollWidth2 = rootRef.current.offsetWidth - rootRef.current.clientWidth;

        setMeasurements({
          gridHeight: rootRef.current.offsetHeight,
          chartWidth: chartRect.width,
          chartHeight: chartRect.height,
          textWidth: textRect.width,
          scrollWidth: _scrollWidth2
        });
      }
    };

    window.addEventListener('resize', updateWindowWidth);
    return function () {
      window.removeEventListener('resize', updateWindowWidth);
    };
  }, []);
  useEffect(function () {
    if (rootRef && rootRef.current && highlighted !== undefined) {
      var rootNode = rootRef.current;
      var highlightedElm = rootNode.querySelector("#" + highlightedIdName);

      if (highlightedElm) {
        highlightedElm.scrollIntoView({
          behavior: "smooth",
          block: "center"
        });
      } else if (onHighlightError) {
        onHighlightError(highlighted);
      }
    }
  }, [rootRef, highlighted]);
  var orderedData = orderBy(data, ['value'], 'desc');
  var maxValue = 100;
  var totalTopValues = gridHeight < 500 ? 10 : 20;
  var rowHeight = gridHeight ? 1 / totalTopValues * gridHeight : 0;
  var axisIncrement = numberOfXAxisTicks ? maxValue / numberOfXAxisTicks : 25;
  var totalAxisValues = maxValue / axisIncrement;
  var axisFontSize;

  if (chartWidth < gridHeight) {
    axisFontSize = "clamp(0.55rem, " + chartWidth * 0.025 + "px, 0.7rem)";
  } else {
    axisFontSize = "clamp(0.55rem, " + gridHeight * 0.025 + "px, 0.7rem)";
  }

  var axisWidth = chartWidth / totalAxisValues;
  var axisLines = [];

  for (var i = 0; i < totalAxisValues + 1; i++) {
    var value = axisIncrement * i;

    if (value <= maxValue) {
      var formatted = formatValue ? formatValue(value) : value;
      axisLines.push(React.createElement(AxisValue, {
        key: 'axis-line-right-' + i,
        style: {
          width: axisWidth
        },
        className: 'react-comparison-bar-chart-axis-value'
      }, React.createElement(AxisText, {
        "$dynamicFont": axisFontSize
      }, formatted), React.createElement(AxisLine, null)));
    }
  }

  var rows = [];
  orderedData.forEach(function (d, i) {
    rows.push(React.createElement(Row$1, {
      key: d.id,
      d: d,
      rowHeight: rowHeight,
      gridHeight: gridHeight,
      max: maxValue,
      onRowHover: onRowHover,
      highlighted: highlighted,
      textWidth: textWidth,
      chartWidth: chartWidth,
      centerLineValue: centerLineValue,
      lessThan1: d.value < centerLineValue
    }));
    var midlineFontSize = "clamp(0.75rem, " + chartHeight * 0.025 + "px, 0.875rem)";

    if (d && d.value >= centerLineValue && (!orderedData[i + 1] || orderedData[i + 1].value < centerLineValue)) {
      rows.push(React.createElement(BufferRow, {
        key: 'vertical-bar-chart-midline',
        style: {
          height: rowHeight,
          visibility: chartWidth ? undefined : 'hidden'
        }
      }, React.createElement(Midline, {
        style: {
          height: rowHeight / 2
        }
      }, React.createElement(Cell, {
        style: {
          height: rowHeight,
          width: "calc(" + textWidth + "px + 0.75rem)",
          backgroundColor: '#fff'
        }
      }), React.createElement(MidlineOverText, {
        "$dynamicFont": midlineFontSize
      }, overMideLineLabel, " \u2191"), React.createElement(MidlineUnderText, {
        "$dynamicFont": midlineFontSize
      }, underMideLineLabel, " \u2193")), React.createElement(Cell, {
        style: {
          height: rowHeight,
          width: "calc(" + textWidth + "px + 0.75rem)",
          borderRight: 'solid 1px #333'
        }
      })));
    } else if (d && d.value < centerLineValue && i === 0) {
      rows.unshift(React.createElement(BufferRow, {
        key: 'vertical-bar-chart-midline',
        style: {
          height: rowHeight,
          visibility: chartWidth ? undefined : 'hidden'
        }
      }, React.createElement(Midline, {
        style: {
          height: rowHeight / 2
        }
      }, React.createElement(Cell, {
        style: {
          height: rowHeight,
          width: "calc(" + textWidth + "px + 0.75rem)",
          backgroundColor: '#fff'
        }
      }), React.createElement(MidlineOverText, {
        "$dynamicFont": midlineFontSize
      }, overMideLineLabel, " \u2191"), React.createElement(MidlineUnderText, {
        "$dynamicFont": midlineFontSize
      }, underMideLineLabel, " \u2193")), React.createElement(Cell, {
        style: {
          height: rowHeight,
          width: "calc(" + textWidth + "px + 0.75rem)",
          borderRight: 'solid 1px #333'
        }
      })));
    }
  });
  var remainingSpace = totalTopValues - rows.length;

  if (remainingSpace > 0) {
    for (var _i = 0; _i < remainingSpace - 1; _i++) {
      var d = {
        id: 'empty-row-buffer-' + _i,
        title: '',
        value: 0,
        color: 'transparent'
      };
      rows.push(React.createElement(Row$1, {
        key: d.id,
        d: d,
        rowHeight: rowHeight,
        gridHeight: gridHeight,
        max: maxValue,
        onRowHover: undefined,
        highlighted: undefined,
        textWidth: textWidth,
        chartWidth: chartWidth,
        centerLineValue: centerLineValue,
        lessThan1: true
      }));
    }
  }

  var axisTitle = axisLabel ? React.createElement(AxisTitle, {
    style: {
      width: chartWidth,
      right: 0,
      textAlign: 'right'
    },
    className: 'react-comparison-bar-chart-axis-title'
  }, axisLabel) : null;
  var buffer = {
    paddingRight: overflowPadding + 'rem',
    marginRight: scrollWidth
  };
  var scrollDown = rows.length > 39 ? React.createElement(ScrollDownText, {
    style: {
      visibility: chartWidth ? undefined : 'hidden'
    },
    "$dynamicFont": "clamp(0.65rem, " + chartWidth * 0.02 + "px, 0.75rem)"
  }, React.createElement(ScrollDownArrow, null, "\u2193"), scrollDownText) : null;
  return React.createElement(Container, {
    style: _extends({}, buffer),
    className: 'react-comparison-bar-chart-root-container'
  }, React.createElement(TitleRoot, {
    style: _extends({
      width: chartWidth,
      visibility: chartWidth ? undefined : 'hidden',
      marginLeft: undefined,
      right: 0,
      left: undefined
    }, buffer),
    "$dynamicFont": "clamp(0.65rem, " + chartWidth * 0.02 + "px, 0.875rem)"
  }, React.createElement(AxisLines, {
    style: {
      height: gridHeight
    }
  }, axisLines)), React.createElement(ChartContainer, null, React.createElement(Grid, {
    ref: rootRef,
    style: {
      gridTemplateRows: 'repeat(${totalValues}, auto)',
      gridTemplateColumns: 'clamp(120px, 300px, 29%) 0.75rem 1fr'
    },
    className: 'react-comparison-bar-chart-grid'
  }, React.createElement(Cell, {
    ref: textRef
  }), React.createElement(Cell, null), React.createElement(Cell, {
    ref: chartRef
  }), React.createElement(ChartBlock, null, React.createElement(BufferRow, {
    style: {
      height: rowHeight,
      position: 'sticky',
      top: '0',
      background: '#fff'
    }
  }), rows), scrollDown)), React.createElement(ChartOverlayContainer, {
    style: _extends({
      width: chartWidth,
      visibility: chartWidth ? undefined : 'hidden',
      marginLeft: undefined,
      right: 0,
      left: undefined
    }, buffer)
  }, React.createElement(AxisLines, {
    style: {
      height: gridHeight,
      width: chartWidth
    }
  }, axisTitle, React.createElement(CenterLine, {
    style: {
      left: centerLineValue + '%'
    }
  }, React.createElement(CenterLineLabel, {
    "$dynamicFont": "clamp(0.5rem, " + chartWidth * 0.023 + "px, 0.875rem)",
    className: 'react-comparison-bar-chart-axis-title'
  }, React.createElement(CenterLineBackground, null, centerLineLabel))))));
};

var VerticalBarChart = function VerticalBarChart(props) {
  return React.createElement(Root, Object.assign({}, props));
};

export default VerticalBarChart;
//# sourceMappingURL=index.modern.js.map
